# If any of the following macros should be set otherwise,
# you can wrap any of them with the following conditions:
# - %%if 0%%{centos} == 7
# - %%if 0%%{?rhel} == 7
# - %%if 0%%{?fedora} == 23
# Or just test for particular distribution:
# - %%if 0%%{centos}
# - %%if 0%%{?rhel}
# - %%if 0%%{?fedora}
#
# Be aware, on centos, both %%rhel and %%centos are set. If you want to test
# rhel specific macros, you can use %%if 0%%{?rhel} && 0%%{?centos} == 0 condition.
# (Don't forget to replace double percentage symbol with single one in order to apply a condition)

# You need to bootstrap this to build it the first time
%global bootstrap 1

# Generate devel rpm
%global with_devel 1
# Build project from bundled dependencies
%global with_bundled 0
# Build with debug info rpm
%global with_debug 1
# Run tests in check section
%if %{bootstrap}
# Cannot run the tests because of a dep loop between minio and mc
%global with_check 0
%else
%global with_check 1
%endif
# Generate unit-test rpm
%global with_unit_test 1

%if 0%{?with_debug}
%global _dwz_low_mem_die_limit 0
%else
%global debug_package   %{nil}
%endif

%if ! 0%{?gobuild:1}
%define gobuild(o:) go build -ldflags "${LDFLAGS:-} -B 0x$(head -c20 /dev/urandom|od -An -tx1|tr -d ' \\n')" -a -v -x %{?**};
%endif

%global provider        github
%global provider_tld    com
%global project         minio
%global repo            minio
# https://github.com/minio/minio
%global provider_prefix %{provider}.%{provider_tld}/%{project}/%{repo}
%global import_path     %{provider_prefix}
%global commit          533338bdebd1ca082793684ea418b9d23738dcf4
%global shortcommit     %(c=%{commit}; echo ${c:0:7})

Name:           golang-%{provider}-%{project}-%{repo}
Version:        0
Release:        0.1.git%{shortcommit}%{?dist}
Summary:        An object storage server compatible with Amazon S3
License:        ASL 2.0
URL:            https://%{provider_prefix}
Source0:        https://%{provider_prefix}/archive/%{commit}/%{repo}-%{shortcommit}.tar.gz

# e.g. el6 has ppc64 arch without gcc-go, so EA tag is required
ExclusiveArch:  %{?go_arches:%{go_arches}}%{!?go_arches:%{ix86} x86_64 aarch64 %{arm}}
# If go_compiler is not set to 1, there is no virtual provide. Use golang instead.
BuildRequires:  %{?go_compiler:compiler(go-compiler)}%{!?go_compiler:golang}

%if ! 0%{?with_bundled}
# Remaining dependencies not included in main packages
BuildRequires: golang(github.com/minio/redigo/redis)
BuildRequires: golang(github.com/minio/dsync)
BuildRequires: golang(golang.org/x/crypto/bcrypt)
BuildRequires: golang(github.com/fatih/color)
BuildRequires: golang(github.com/streadway/amqp)
BuildRequires: golang(github.com/gorilla/mux)
BuildRequires: golang(github.com/Sirupsen/logrus)
BuildRequires: golang(github.com/skyrings/skyring-common/tools/uuid)
BuildRequires: golang(github.com/gorilla/rpc/v2)
BuildRequires: golang(github.com/nats-io/go-nats-streaming)
BuildRequires: golang(github.com/dustin/go-humanize)
BuildRequires: golang(github.com/klauspost/reedsolomon)
BuildRequires: golang(github.com/dgrijalva/jwt-go)
BuildRequires: golang(github.com/rs/cors)
BuildRequires: golang(github.com/tidwall/gjson)
BuildRequires: golang(github.com/gorilla/handlers)
BuildRequires: golang(github.com/lib/pq)
BuildRequires: golang(github.com/nats-io/nats)
BuildRequires: golang(github.com/minio/sha256-simd)
BuildRequires: golang(github.com/minio/go-homedir)
%if !%{bootstrap}
BuildRequires: golang(github.com/minio/mc/pkg/console)
%endif
BuildRequires: golang(github.com/gorilla/rpc/v2/json2)
BuildRequires: golang(github.com/minio/blake2b-simd)
BuildRequires: golang(github.com/fatih/structs)
BuildRequires: golang(github.com/pkg/profile)
BuildRequires: golang(gopkg.in/olivere/elastic.v3)
BuildRequires: golang(github.com/elazarl/go-bindata-assetfs)
BuildRequires: golang(github.com/dgrijalva/jwt-go/request)
BuildRequires: golang(github.com/minio/cli)
BuildRequires: golang(github.com/cheggaaa/pb)
BuildRequires: golang(github.com/hashicorp/go-version)
%endif

%description
%{summary}

%if 0%{?with_devel}
%package devel
Summary:       %{summary}
BuildArch:     noarch

%if 0%{?with_check} && ! 0%{?with_bundled}
BuildRequires: golang(github.com/Sirupsen/logrus)
BuildRequires: golang(github.com/cheggaaa/pb)
BuildRequires: golang(github.com/dgrijalva/jwt-go)
BuildRequires: golang(github.com/dgrijalva/jwt-go/request)
BuildRequires: golang(github.com/dustin/go-humanize)
BuildRequires: golang(github.com/elazarl/go-bindata-assetfs)
BuildRequires: golang(github.com/fatih/color)
BuildRequires: golang(github.com/fatih/structs)
BuildRequires: golang(github.com/gorilla/handlers)
BuildRequires: golang(github.com/gorilla/mux)
BuildRequires: golang(github.com/gorilla/rpc/v2)
BuildRequires: golang(github.com/gorilla/rpc/v2/json2)
BuildRequires: golang(github.com/hashicorp/go-version)
BuildRequires: golang(github.com/klauspost/reedsolomon)
BuildRequires: golang(github.com/lib/pq)
BuildRequires: golang(github.com/minio/blake2b-simd)
BuildRequires: golang(github.com/minio/cli)
BuildRequires: golang(github.com/minio/dsync)
BuildRequires: golang(github.com/minio/go-homedir)
BuildRequires: golang(github.com/minio/mc/pkg/console)
BuildRequires: golang(github.com/minio/redigo/redis)
BuildRequires: golang(github.com/minio/sha256-simd)
BuildRequires: golang(github.com/nats-io/go-nats-streaming)
BuildRequires: golang(github.com/nats-io/nats)
BuildRequires: golang(github.com/pkg/profile)
BuildRequires: golang(github.com/rs/cors)
BuildRequires: golang(github.com/skyrings/skyring-common/tools/uuid)
BuildRequires: golang(github.com/streadway/amqp)
BuildRequires: golang(github.com/tidwall/gjson)
BuildRequires: golang(golang.org/x/crypto/bcrypt)
BuildRequires: golang(gopkg.in/olivere/elastic.v3)
%endif

Requires:      golang(github.com/Sirupsen/logrus)
Requires:      golang(github.com/cheggaaa/pb)
Requires:      golang(github.com/dgrijalva/jwt-go)
Requires:      golang(github.com/dgrijalva/jwt-go/request)
Requires:      golang(github.com/dustin/go-humanize)
Requires:      golang(github.com/elazarl/go-bindata-assetfs)
Requires:      golang(github.com/fatih/color)
Requires:      golang(github.com/fatih/structs)
Requires:      golang(github.com/gorilla/handlers)
Requires:      golang(github.com/gorilla/mux)
Requires:      golang(github.com/gorilla/rpc/v2)
Requires:      golang(github.com/gorilla/rpc/v2/json2)
Requires:      golang(github.com/hashicorp/go-version)
Requires:      golang(github.com/klauspost/reedsolomon)
Requires:      golang(github.com/lib/pq)
Requires:      golang(github.com/minio/blake2b-simd)
Requires:      golang(github.com/minio/cli)
Requires:      golang(github.com/minio/dsync)
Requires:      golang(github.com/minio/go-homedir)
%if !%{bootstrap}
Requires:      golang(github.com/minio/mc/pkg/console)
%endif
Requires:      golang(github.com/minio/redigo/redis)
Requires:      golang(github.com/minio/sha256-simd)
Requires:      golang(github.com/nats-io/go-nats-streaming)
Requires:      golang(github.com/nats-io/nats)
Requires:      golang(github.com/pkg/profile)
Requires:      golang(github.com/rs/cors)
Requires:      golang(github.com/skyrings/skyring-common/tools/uuid)
Requires:      golang(github.com/streadway/amqp)
Requires:      golang(github.com/tidwall/gjson)
Requires:      golang(golang.org/x/crypto/bcrypt)
Requires:      golang(gopkg.in/olivere/elastic.v3)

Provides:      golang(%{import_path}/browser) = %{version}-%{release}
Provides:      golang(%{import_path}/cmd) = %{version}-%{release}
Provides:      golang(%{import_path}/pkg/bpool) = %{version}-%{release}
Provides:      golang(%{import_path}/pkg/disk) = %{version}-%{release}
Provides:      golang(%{import_path}/pkg/lock) = %{version}-%{release}
Provides:      golang(%{import_path}/pkg/madmin) = %{version}-%{release}
Provides:      golang(%{import_path}/pkg/mimedb) = %{version}-%{release}
Provides:      golang(%{import_path}/pkg/objcache) = %{version}-%{release}
Provides:      golang(%{import_path}/pkg/quick) = %{version}-%{release}
Provides:      golang(%{import_path}/pkg/safe) = %{version}-%{release}
Provides:      golang(%{import_path}/pkg/sys) = %{version}-%{release}
Provides:      golang(%{import_path}/pkg/wildcard) = %{version}-%{release}

%description devel
%{summary}

This package contains library source intended for
building other packages which use import path with
%{import_path} prefix.
%endif

%if 0%{?with_unit_test} && 0%{?with_devel}
%package unit-test-devel
Summary:         Unit tests for %{name} package
%if 0%{?with_check}
#Here comes all BuildRequires: PACKAGE the unit tests
#in %%check section need for running
%endif

# test subpackage tests code from devel subpackage
Requires:        %{name}-devel = %{version}-%{release}

%if 0%{?with_check} && ! 0%{?with_bundled}
BuildRequires: golang(gopkg.in/check.v1)
%endif

Requires:      golang(gopkg.in/check.v1)

%description unit-test-devel
%{summary}

This package contains unit tests for project
providing packages with %{import_path} prefix.
%endif

%prep
%setup -q -n %{repo}-%{commit}

%build
mkdir -p src/%{provider}.%{provider_tld}/%{project}
ln -s ../../../ src/%{import_path}

%if ! 0%{?with_bundled}
export GOPATH=$(pwd):%{gopath}
%else
# No dependency directories so far
export GOPATH=$(pwd):%{gopath}
%endif

%gobuild -o bin/minio %{import_path}/
#%gobuild -o bin/buildscripts %{import_path}/buildscripts
#%gobuild -o bin/pkg/madmin/examples %{import_path}/pkg/madmin/examples
#%gobuild -o bin/pkg/mimedb/util %{import_path}/pkg/mimedb/util

%install
install -d -p %{buildroot}%{_bindir}
install -p -m 0755 bin/minio %{buildroot}%{_bindir}
#install -p -m 0755 bin/buildscripts %{buildroot}%{_bindir}
#install -p -m 0755 bin/pkg/madmin/examples %{buildroot}%{_bindir}
#install -p -m 0755 bin/pkg/mimedb/util %{buildroot}%{_bindir}

# source codes for building projects
%if 0%{?with_devel}
install -d -p %{buildroot}/%{gopath}/src/%{import_path}/
echo "%%dir %%{gopath}/src/%%{import_path}/." >> devel.file-list
# find all *.go but no *_test.go files and generate devel.file-list
for file in $(find . \( -iname "*.go" -or -iname "*.s" \) \! -iname "*_test.go" | grep -v "vendor") ; do
    dirprefix=$(dirname $file)
    install -d -p %{buildroot}/%{gopath}/src/%{import_path}/$dirprefix
    cp -pav $file %{buildroot}/%{gopath}/src/%{import_path}/$file
    echo "%%{gopath}/src/%%{import_path}/$file" >> devel.file-list

    while [ "$dirprefix" != "." ]; do
        echo "%%dir %%{gopath}/src/%%{import_path}/$dirprefix" >> devel.file-list
        dirprefix=$(dirname $dirprefix)
    done
done
%endif

# testing files for this project
%if 0%{?with_unit_test} && 0%{?with_devel}
install -d -p %{buildroot}/%{gopath}/src/%{import_path}/
# find all *_test.go files and generate unit-test-devel.file-list
for file in $(find . -iname "*_test.go" | grep -v "vendor") ; do
    dirprefix=$(dirname $file)
    install -d -p %{buildroot}/%{gopath}/src/%{import_path}/$dirprefix
    cp -pav $file %{buildroot}/%{gopath}/src/%{import_path}/$file
    echo "%%{gopath}/src/%%{import_path}/$file" >> unit-test-devel.file-list

    while [ "$dirprefix" != "." ]; do
        echo "%%dir %%{gopath}/src/%%{import_path}/$dirprefix" >> devel.file-list
        dirprefix=$(dirname $dirprefix)
    done
done
%endif

%if 0%{?with_devel}
sort -u -o devel.file-list devel.file-list
%endif

%check
%if 0%{?with_check} && 0%{?with_unit_test} && 0%{?with_devel}
%if ! 0%{?with_bundled}
export GOPATH=%{buildroot}/%{gopath}:%{gopath}
%else
# Since we aren't packaging up the vendor directory we need to link
# back to it somehow. Hack it up so that we can add the vendor
# directory from BUILD dir as a gopath to be searched when executing
# tests from the BUILDROOT dir.
ln -s ./ ./vendor/src # ./vendor/src -> ./vendor

export GOPATH=%{buildroot}/%{gopath}:$(pwd)/vendor:%{gopath}
%endif

%if ! 0%{?gotest:1}
%global gotest go test
%endif

%gotest %{import_path}/cmd
%gotest %{import_path}/pkg/bpool
%gotest %{import_path}/pkg/disk
%gotest %{import_path}/pkg/lock
%gotest %{import_path}/pkg/madmin
%gotest %{import_path}/pkg/mimedb
%gotest %{import_path}/pkg/objcache
%gotest %{import_path}/pkg/quick
%gotest %{import_path}/pkg/safe
%gotest %{import_path}/pkg/sys
%gotest %{import_path}/pkg/wildcard
%endif

#define license tag if not already defined
%{!?_licensedir:%global license %doc}

%files
%license LICENSE
%doc README.md MAINTAINERS.md CONTRIBUTING.md README_ZH.md
%{_bindir}/*
#%{_bindir}/buildscripts
#%{_bindir}/pkg/madmin/examples
#%{_bindir}/pkg/mimedb/util

%if 0%{?with_devel}
%files devel -f devel.file-list
%license LICENSE
%doc README.md MAINTAINERS.md CONTRIBUTING.md README_ZH.md
%dir %{gopath}/src/%{provider}.%{provider_tld}/%{project}
%endif

%if 0%{?with_unit_test} && 0%{?with_devel}
%files unit-test-devel -f unit-test-devel.file-list
%license LICENSE
%doc README.md MAINTAINERS.md CONTRIBUTING.md README_ZH.md
%endif

%changelog
* Sat Feb 04 2017 Tom Callaway <spot@fedoraproject.org> - 0-0.1.git533338b
- First package for Fedora

